/* Copyright (c) 2009 Ondrej Donek.
   See the file LICENSE.txt for licensing information. */

// Some constants
if(typeof(CC) == "undefined") const CC = Components.classes;  
if(typeof(CI) == "undefined") const CI = Components.interfaces; 
if(typeof(CU) == "undefined") const CU = Components.utils;

if(typeof(XUL_NS) == "undefined") 
	const XUL_NS  = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

// Our namespace
if(typeof(extensions) == "undefined") var extensions = {};
if(typeof(extensions.tbnotes) == "undefined") extensions.tbnotes = {};

// Some URLs to icons
const TBNOTES_MSGHASNOTE_ICON_URL = "chrome://tbnotes/skin/icons/note-flag16x16.png";
const TBNOTES_MSGHASNONOTE_ICON_URL = "chrome://tbnotes/skin/icons/note-no-flag16x16.png";


/**
 * Main object that contains all main functionality of tbNotes.
 * 
 * @type Object
 */
(function() {
	
	// ======================================================================
	// Private members
	
	/**
	 * This object serves all about storage
	 * 
	 * @private
	 * @link http://developer.mozilla.org/en/Extensions/Thunderbird/HowTos/Common_Thunderbird_Extension_Techniques/Use_SQLite
	 * @type Object
	 */
	var db = {
		
		/**
		 * Holds database connection
		 * @type Components.interfaces.mozIStorageConnection
		 */
		mDbConnection: null,
		
		/**
		 * Holds object describing our database schema
		 * @type Object
		 */
		mDbSchema: {
			tables: {
				tbn_notes:
					"id				INTEGER PRIMARY KEY, \
					 category_id	INTEGER NOT NULL, \
				     created		DATETIME NOT NULL, \
					 updated		DATETIME NOT NULL, \
					 title			VARCHAR NOT NULL, \
					 body			TEXT, \
					 deleted		INTEGER DEFAULT 0",
				tbn_categories:
					"id				INTEGER PRIMARY KEY, \
					 parent_id  	INTEGER, \
					 name       	VARCHAR NOT NULL, \
					 description	VARCHAR "
			}
		},
		
		/**
		 * Initialize SQLite database
		 * 
		 * @return void
		 */
		onLoad: function() {
			this.initialized = true;
			this._dbInit();
		},
		
		/**
		 * Initialize database and establish the connection
		 * 
		 * @returns boolean
		 */
		_dbInit: function() {
			var dirService = CC["@mozilla.org/file/directory_service;1"].  
					getService(CI.nsIProperties);  

			var dbFile = dirService.get("ProfD", CI.nsIFile);  
					dbFile.append("tbnotes.sqlite");  

			var dbService = CC["@mozilla.org/storage/service;1"].  
					getService(CI.mozIStorageService);  

			var dbConnection = (!dbFile.exists())
					? this._dbCreate(dbService, dbFile)
					: dbService.openDatabase(dbFile);

			this.mDbConnection = dbConnection;
		},
		
		/**
		 * Create database
		 * 
		 * @return {Components.interfaces.mozIStorageConnection}
		 */
		_dbCreate: function(aDbService, aDbFile) {
			var dbConnection = aDbService.openDatabase(aDbFile);
			this._dbCreateTables(dbConnection);
			
			return dbConnection;
		},
		
		/**
		 * Creates tables
		 * 
		 * @return void
		 */
		_dbCreateTables: function(aDbConnection) {
			for(var name in this.mDbSchema.tables) {
				aDbConnection.createTable(name, this.mDbSchema.tables[name]);
			}
		}
	};
	
	// ======================================================================
	// Event handlers
	
	/**
	 * Called when Thunderbird is loaded
	 * 
	 * @return void
	 */
	this.onLoad = function() {
		// Initialize tbNotes SQLite storage
		db.onLoad();
		
		// Add new col to the main Thunderbird's messages tree
		var obSrv = CC["@mozilla.org/observer-service;1"].
				getService(CI.nsIObserverService);
		
		obSrv.addObserver(gTbNotesCreateDbObserver, "MsgCreateDBView", false);
	};
	
	/**
	 * Performs command 
	 * 
	 * @param aCmdName {string}
	 * @return void
	 */
	this.doCommand = function(aCmdName) {
		switch(aCmdName) {
			case "add_note":
				// TODO Implement command "add_note"!
				alert("Not implemented yet!");
				break;
			
			default:
				CU.reportError("extensions.tbnotes.doCommand: Unknown command ['" + aCmdName + "']!");
				break;
		}
	};
	
	// ======================================================================
	// Normal methods
	
	/**
	 * Returns TRUE if message with given key has any note attached or FALSE.
	 * 
	 * @param aKey {????}
	 * @return boolean
	 */
	this.checkNoteForKey = function(aKey) {
		// TODO Implement this!!!
		CU.reportError("Given key: " + aKey);
		
		return false;
	};
	
}).apply(extensions.tbnotes);
